/***********************************************************************\
*                          __   ______  _____ __                        *
*                   __ _  / /__<  / _ )/ __(_) /__ ___                  *
*                  /  ' \/  '_// / _  / _// / / -_|_-<                  *
*                 /_/_/_/_/\_\/_/____/_/ /_/_/\__/___/                  *
*                                                                       *
*  -------------------------------------------------------------------  *
*  This file is a part of mk1BFiles                                     *
*                                                                       *
*  mk1BFiles is free software; you can redistribute it and/or           *
*  modify it under the terms of the GNU General Public License          *
*  as published by the Free Software Foundation; either version 3       *
*  of the License, or (at your option) any later version.               *
*                                                                       *
*  This program is distributed in the hope that it will be useful,      *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
*  GNU General Public License for more details.                         *
*                                                                       *
*  You should have received a copy of the GNU General Public License    *
*  along with this program; if not, write to the Free Software          *
*  Foundation, Inc., 51 Franklin Street, Fifth Floor,                   *
*  Boston, MA  02110-1301, USA.                                         *
*                                                                       *
*  -------------------------------------------------------------------  *
*  Copyright (C) 2010, Clercin guillaume <clercin.guillaume@gmail.com>  *
*  Last modified: Mon, 06 Sep 2010 17:24:38 +0200                       *
\***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>

#include "display.h"
#include "timer.h"

static void report3(const char * r, unsigned long long current);

static struct timeval begin;
static struct timeval end;
static unsigned long long maxVal = 100;

void report(const char * r) {
	report3(r, maxVal);
}

void report2(const char * r, unsigned long long current) {
	if (!current)
		return;

	stopTimer();

	report3(r, current);
}

void report3(const char * r, unsigned long long current) {
	double inter = difftime(end.tv_sec, begin.tv_sec) + difftime(end.tv_usec, begin.tv_usec) / 1000000;
	double pct = ((double) current) / maxVal;
	double total = inter / pct - inter;
	pct *= 100;

	char buffer[256];
	strcpy(buffer, r);

	char * ptr = 0;
	if (ptr = strstr(buffer, "%E")) {
		unsigned long long tmpTotal = total;
		unsigned int totalMilli = 1000 * (total - tmpTotal);
		short totalSecond = tmpTotal % 60;
		tmpTotal /= 60;
		short totalMinute = tmpTotal % 60;
		unsigned int totalHour = tmpTotal / 60;

		char tmp[16];
		snprintf(tmp, 16, "%u:%02u:%02u.%03u", totalHour, totalMinute, totalSecond, totalMilli);
		int length = strlen(tmp);

		do {
			if (length != 2)
				memmove(ptr + length, ptr + 2, strlen(ptr + 2) + 1);
			memcpy(ptr, tmp, length);
		} while (ptr = strstr(ptr + 1, "%E"));
	}

	if (ptr = strstr(buffer, "%L")) {
		unsigned long long tmpInter = inter;
		unsigned int interMilli = 1000 * (inter - tmpInter);
		short interSecond = tmpInter % 60;
		tmpInter /= 60;
		short interMinute = tmpInter % 60;
		unsigned int interHour = tmpInter / 60;

		char tmp[16];
		snprintf(tmp, 16, "%u:%02u:%02u.%03u", interHour, interMinute, interSecond, interMilli);
		int length = strlen(tmp);

		do {
			if (length != 2)
				memmove(ptr + length, ptr + 2, strlen(ptr + 2) + 1);
			memcpy(ptr, tmp, length);
		} while (ptr = strstr(ptr + 1, "%L"));
	}

	if (ptr = strstr(buffer, "%p")) {
		char tmp[16];
		snprintf(tmp, 16, "% 6.3f%%%%", pct);
		int length = strlen(tmp);

		do {
			if (length != 2)
				memmove(ptr + length, ptr + 2, strlen(ptr + 2) + 1);
			memcpy(ptr, tmp, length);
		} while (ptr = strstr(ptr + length, "%p"));
	}

	ptr = buffer;
	while (ptr = strchr(ptr, '%')) {
		int s = 0, t = 0;
		if (sscanf(ptr + 1, "%db%n", &s, &t) > 0) {
			t++;
			int p = s * current / maxVal;

			if (s != t)
				memmove(ptr + s, ptr + t, strlen(ptr + t) + 1);
			memset(ptr, '#', p);
			memset(ptr + p, '.', s - p);
		}
		ptr++;
	}

	mkPrint(buffer);
}

void restartTimer() {
	gettimeofday(&begin, 0);
}

void setMax(unsigned long long max) {
	maxVal = max;
}

void stopTimer() {
	gettimeofday(&end, 0);
}

