/***********************************************************************\
*                          __   ______  _____ __                        *
*                   __ _  / /__<  / _ )/ __(_) /__ ___                  *
*                  /  ' \/  '_// / _  / _// / / -_|_-<                  *
*                 /_/_/_/_/\_\/_/____/_/ /_/_/\__/___/                  *
*                                                                       *
*  -------------------------------------------------------------------  *
*  This file is a part of mk1BFiles                                     *
*                                                                       *
*  mk1BFiles is free software; you can redistribute it and/or           *
*  modify it under the terms of the GNU General Public License          *
*  as published by the Free Software Foundation; either version 3       *
*  of the License, or (at your option) any later version.               *
*                                                                       *
*  This program is distributed in the hope that it will be useful,      *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
*  GNU General Public License for more details.                         *
*                                                                       *
*  You should have received a copy of the GNU General Public License    *
*  along with this program; if not, write to the Free Software          *
*  Foundation, Inc., 51 Franklin Street, Fifth Floor,                   *
*  Boston, MA  02110-1301, USA.                                         *
*                                                                       *
*  -------------------------------------------------------------------  *
*  Copyright (C) 2010, Clercin guillaume <clercin.guillaume@gmail.com>  *
*  Last modified: Mon, 06 Sep 2010 17:22:30 +0200                       *
\***********************************************************************/

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/resource.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <time.h>
#include <unistd.h>

#include "display.h"
#include "util.h"

void convertSize(char * buffer, unsigned int bufferSize, long long size) {
	unsigned short mult = 0;
	double tsize = size;

	while (tsize > 1024 && mult < 4) {
		tsize /= 1024;
		mult++;
	}

	switch (mult) {
		case 0:
			snprintf(buffer, bufferSize, "%lld Bytes", size);
			break;
		case 1:
			snprintf(buffer, bufferSize, "%.1f KBytes", tsize);
			break;
		case 2:
			snprintf(buffer, bufferSize, "%.2f MBytes", tsize);
			break;
		case 3:
			snprintf(buffer, bufferSize, "%.3f GBytes", tsize);
			break;
		default:
			snprintf(buffer, bufferSize, "%.4f TBytes", tsize);
	}
}

void convertTime(char * buffer, unsigned int bufferSize, struct timeval * tv) {
	if (!buffer || bufferSize < 4 || !tv)
		return;

	long int total = tv->tv_sec;
	short second = total % 60;
	total /= 60;
	short minute = total % 60;
	total /= 60;
	short hour = total % 24;
	short mili = tv->tv_usec / 1000;

	snprintf(buffer, bufferSize, "%d:%02d:%02d.%03d", hour, minute, second, mili);
}

void getCurrentTime(char * buffer, unsigned int bufferSize) {
	struct timeval curTime;
	struct tm curTime2;

	gettimeofday(&curTime, 0);
	localtime_r(&(curTime.tv_sec), &curTime2);
	strftime(buffer, bufferSize, "%F %T", &curTime2);
}

void run(const char * com, const char ** params) {
	char command[256];
	char * path = getenv("PATH");
	short found = 0;

	if (path) {
		char * cPath = strdup(path);

		char * ptr = 0;
		for (ptr = strtok(cPath, ":"); ptr; ptr = strtok(0, ":")) {
			snprintf(command, 256, "%s/%s", ptr, com);
			if (!access(command, R_OK | X_OK)) {
				found = 1;
				break;
			}
		}

		free(cPath);
	}

	if (!found) {
		mkPrintf("Command not found '%s'\n", com);
		return;
	}

	int pid = fork();

	if (pid > 0) {
		int status = 0;
		struct rusage usage;
		wait4(pid, &status, 0, &usage);

		char cpuUser[24];
		char cpuSystem[24];
		char cStatus[32];
		char maxRss[24];

		if (WIFEXITED(status))
			snprintf(cStatus, 32, "ok, returned value: %d", WEXITSTATUS(status));
		else if (WIFSIGNALED(status))
			snprintf(cStatus, 32, "kill by signal, signal: %d", WTERMSIG(status));

		convertTime(cpuUser, 24, &(usage.ru_utime));
		convertTime(cpuSystem, 24, &(usage.ru_stime));
		convertSize(maxRss, 24, usage.ru_maxrss << 10);

		mkPrintf("Stat: %s (pid: %d) status: %s\n", com, pid, cStatus);
		mkPrintf("Cpu usage: user: %s, system: %s\n", cpuUser, cpuSystem);
		mkPrintf("Max rss: %s, page fault: major: %ld, minor: %ld\n\n", maxRss, usage.ru_majflt, usage.ru_minflt);

	} else if (pid == 0) {
		int fd = open("/dev/null", O_WRONLY);

		dup2(fd, 1);
		dup2(fd, 2);
		close(0);

		if (fd > 2 || fd == 0)
			close(fd);

		params[0] = com;
		execvp(command, (char * const *) params);

		exit(3);
	} else {
		mkPrintf("Failed to fork\n", com);
	}
}

